subroutine adaptspec(nfft,ne,kspec,dvar,evalu1,yk,	&
                    spec,se,wt,err)

!
!  Modified
!	German Prieto
!	July 2005
!	
!	*** Corrected the estimation of the dofs se(1:ne)
!           (set sum(wt**2,dim=2) = 1.), with maximum wt=1.
!
!  subroutine to calculate adaptively weighted power spectrum
!  and jackknifed 95% confidence limits
!
!  inputs:
!
!    nfft  - number of points in fft's
!    ne    - number of elements in fft to process, normally nfft/2+1
!            for real data and nfft for complex data
!    kspec - number of prolate windows used
!    dvar  - variance of data
!    evalu1- double precision vector containing 1 minus eigenvalues
!    yk    - complex array containing kspec ffts of length nfft
!
!  outputs:
!
!    spec - real vector containing adaptively weighted spectrum
!    se   - real vector containing the number of degrees of freedom
!           for the spectral estimate at each frequency.
!    wt   - real array containing the ne weights for kspec 
!           eigenspectra normalized so that the sum of squares over
!           the kspec eigenspectra is one
!    err  - real array of jackknife error estimates, with 5 and 95%
!           confidence intervals of the spectrum.
!
!
!  calls qtdis
!

!**********************************************************************

   implicit none

!  Input Output

   integer, intent(in) :: nfft, ne, kspec

   real(8), intent(in) :: dvar

   real(8), intent(in), dimension(kspec) :: evalu1
  
   complex(8), intent(in), dimension(nfft,kspec) ::  yk

   real(8), intent(out), dimension(ne) :: spec, se 

   real(8), intent(out), dimension(ne,2) :: err

   real(8), intent(out), dimension(ne,kspec) :: wt

!  Jackknife

   real(4) :: qtdis

   real(8), dimension(ne)  :: sbar, slast, var
   real(8), dimension(ne)  :: lspec, lsjk_mean, bjk

   real(8), dimension(ne,kspec) :: sk, skw, sjk, varjk
   real(8), dimension(ne,kspec) :: lsjk

   real(8), dimension(ne,kspec-1) :: sj, wjk

   real(8), dimension(ne,kspec) :: wt_dofs

!  Other variables

   integer :: i, j, k, ks, nfft2

   real(8) :: rerr

   real(8), dimension(kspec) :: bk
   real(8), dimension(kspec) :: evalu, sqev 

!**********************************************************************

   nfft2=nfft/2

!  Initialize some variables

   bk	 = dvar*evalu1
   evalu = 1.d0 - evalu1
   sqev  = sqrt(evalu)

!
!  step through frequencies and find weighted spectrum iteratively
!  for each
!


   rerr = 9.5d-7	! Previously used r1mach function

!  Generate eigenspectra sk = yk**2

   do i = 1,kspec
      sk(:,i) = (abs(yk(1:ne,i)))**2
   enddo

!  Do iteration

   do i = 1,kspec + 1

      sbar = sum(sk(:,1:2), dim=2)/2.d0

      
      do j = 1,1000

         slast = sbar

	 if (i == kspec+1) then
            do k = 1,kspec
               wt(:,k) = sqev(k)*sbar /( evalu(k)*sbar + bk(k))
	       wt(:,k) = amin1(wt(:,k),1.d0)
               skw(:,k) = wt(:,k)**2 * sk(:,k)   
            enddo
      
            sbar = sum(skw,dim=2)/ sum(wt**2, dim=2)
	    
	    if (j==1000) then
	       spec = sbar
	       write(6,'(a,2E16.7)') 'Did not converge, rerr = ',    &
	            maxval(abs((sbar-slast)/(sbar+slast))),rerr
	       cycle
	    endif
            if(maxval(abs((sbar-slast)/(sbar+slast))) .gt. rerr) then
               cycle
            endif
            
	    spec = sbar
            exit
	 
	 else
            ks = 0
            do k = 1,kspec

	       if (k == i) then
	          cycle
	       endif
	       ks = ks + 1
               wjk(:,ks) = sqev(k)*sbar /( evalu(k)*sbar + bk(k))
               sj(:,ks) = wjk(:,ks)**2 * sk(:,k)   
            enddo
      
            sbar = sum(sj,dim=2)/ sum(wjk**2, dim=2) 

            if (j==1000) then
	       sjk(:,i) = sbar
	       cycle
	    endif

!            print *, wjk(500,:)
 
            if(maxval(abs((sbar-slast)/(sbar+slast))) .gt. rerr) then
               cycle
            endif
 
	    sjk(:,i) = sbar
            exit

	 
         endif
	 
      enddo 

   enddo

!  Jackknife mean (Log S)

   lspec = log(spec)
   lsjk  = log(sjk)

   lsjk_mean = sum(lsjk, dim=2)/dble(kspec)

!  Jackknife Bias estimate (Log S)

   bjk = dble(kspec-1) * (lspec - lsjk_mean)

!  Jackknife Variance Estimate (Log S)

   do i = 1,kspec
      varjk(:,i) = (lsjk(:,i) - lsjk_mean)**2
   enddo
   var = sum(varjk, dim=2) * dble(kspec-1)/dble(kspec)

   do i = 1,ne
      wt_dofs(i,:) = wt(i,:)/sqrt(sum(wt(i,:)**2)/dble(kspec))
   enddo

   wt_dofs = amin1(wt_dofs,1.d0)

   se = 2.d0 * sum(wt_dofs**2, dim=2) 
!   se = 2.d0 * sum(wt**2, dim=2) 

   do i = 1,ne
      if (se(i)<1.d0) then
         write(6,'(a,i5,a,f10.5)') 'dof < 1 ', i, 'th frequency',se(i)
	 stop
      endif
      var(i) = exp( dble(qtdis(0.950, real(se(i)) )) *sqrt(var(i)))
   enddo

!  The setting of the confidence interval is done in mtspec.f90.

   err(:,1) = var
   err(:,2) = var
   
   return
      
end subroutine adaptspec


