subroutine mtcohe ( npts,dt,xi,xj,tbp,kspec,nf,p,                   &
                     freq,spec_i, spec_j,cohe,phase,conf,cohe_ci, phase_ci )  
            

!
!  Construct the coherence spectrum from the yk's and the 
!  weights of the usual multitaper spectrum estimation. 
!  Note this code uses the real(4) multitaper code. 
!
!  Modified
!	German Prieto
!	September 2005
! 
!  INPUT
!
!	npts		integer number of points in time series
! 	dt		real, sampling rate of time series
!	xi(npts)	real, data for first series
!       xj(npts) 	real, data for second series
!	tbp		the time-bandwidth product
!	kspec		integer, number of tapers to use
!	nf		integer, number of freq points in spectrum
!	p		confidence for null hypothesis test
!
!  OUTPUT
!
!	freq(nf)	real vector with frequency bins
!	spec_i(nf)	real vector with spectrum of first series
!	spec_j(nf)	real vector with spectrum of second
!       cohe(nf)	real, coherence of the two series (0 - 1)
!       phase(nf)	the phase at each frequency
!	conf(nf)	p confidence value for each freq.
!	cohe_ci(nf,2)	95% bounds on coherence (not larger than 1)
!	phase_ci(nf,2)  95% bounds on phase estimates
!

!********************************************************************

   use spectra

   implicit none

!  Inputs

   integer, intent(in) :: npts, kspec, nf

   real(4), intent(in) :: dt, tbp
   real(4), intent(in out) :: p

   real(4), dimension(npts), intent(in) :: xi, xj

!  Outputs

   real(4), intent(out), dimension(nf) :: freq, spec_i, spec_j

   real(4), intent(out), dimension(nf) :: cohe, phase, conf
 
   real(4), intent(out), dimension(nf,2) :: cohe_ci, phase_ci

!  spectra and frequency


   real(4), dimension(nf,kspec) :: wt_i, wt_j 

   complex(4), dimension(npts,kspec) :: yk_i, yk_j

   real(4), dimension(2) :: xmean, xvar

!  Coherence freq matrices

   complex(4), dimension(nf,kspec)     :: dyk_i, dyk_j 
   real(4), dimension(nf)              :: dk_i, dk_j 
   complex(4), dimension(nf)           :: spec

!  Confidence terms

   real(4), dimension(nf)       :: se_i, se_j, se

!  Jackknife

   integer :: j
   
   complex(4), dimension(nf,kspec) :: cross
   real(4), dimension(nf,kspec)    :: spec1, spec2

   complex(4), dimension(nf)       :: cross_all
   real(4), dimension(nf)          :: spec1_all, spec2_all
   real(4), dimension(nf)          :: cohe_all, phase_all

   real(4), dimension(nf,kspec)    :: cohejk, Q 
   real(4), dimension(nf,kspec)    :: qvarjk
   real(4), dimension(nf)          :: qmean, qvar, phvar
   real(4), dimension(nf)          :: qtop, qbot, qval

   complex(4), dimension(nf,kspec) :: phasejk
   complex(4), dimension(nf)       :: phmean

   real(4) :: atanh2
   
!  Others

   integer :: i

   real(4), parameter :: pi = 3.14159265358979

!********************************************************************

   write(6,'(a)') 'Calling multitaper coherence code'
   write(6,'(a,2i5)') 'Data points and frequency points', npts, nf

   if (p>=1. .or. p<=0.) then
      p = 0.95
   endif
   write(6,'(a,f10.7)') 'Confidence for null hypothesis testing ', p


   xmean(1) = sum(xi)/real(npts)
   xvar(1) = (sum((xi - xmean(1))**2))/real(npts-1)
   xmean(2) = sum(xj)/real(npts)
   xvar(2) = (sum((xj - xmean(2))**2))/real(npts-1)


!
!  Get the spectrum estimate
!  if xi is xj, then just do one estimate of the spectrum
!

   if (all(xi==xj)) then    
      write(6,'(a)') 'Auto-coherence not applicable'
      stop
   else
      call mtspec( npts,dt,xi,tbp,kspec,nf,freq,          &
                    spec_i,yk=yk_i,wt=wt_i,se=se_i)

      call mtspec( npts,dt,xj,tbp,kspec,nf,freq,          &
    		    spec_j,yk=yk_j,wt=wt_j,se=se_j)
   endif

!
!  Create the spectra (cannot use spec output, normalized different)
!

   spec_i = sum( wt_i**2 * (abs(yk_i(1:nf,:)))**2 , dim=2)/    &
                     sum((wt_i**2),dim=2)

   spec_j = sum( wt_j**2 * (abs(yk_j(1:nf,:)))**2 , dim=2)/    &
                     sum((wt_j**2),dim=2)

   dk_i = sqrt( sum (wt_i(:,:)**2, dim=2) )
   dk_j = sqrt( sum (wt_j(:,:)**2, dim=2) )

   dyk_i(:,:) = wt_i(:,:) * yk_i(1:nf,:)
   dyk_j(:,:) = wt_j(:,:) * yk_j(1:nf,:)

   do i=1,nf

   ! coherence and phase
   
         
      spec(i) = sum ( dyk_i(i,:) * conjg(dyk_j(i,:)) )   &
                               / (dk_i(i)*dk_j(i))

      cohe(i) = (abs(spec(i)))**2 / (spec_i(i)*spec_j(i))

      phase(i) = atan2( aimag(spec(i)), real(spec(i)) ) 

      
   enddo

!  The null hypothesis test with degrees of freedom

   do i = 1,nf
      se(i) = amin1(se_i(i),se_j(i))
      conf(i)  = 1. - ( (1.0-p)**(1./(se(i)/2. -1)) )
   enddo

!  Jackknife estimate for confidence intervals
!  Use the arctanh transformation, close to Gaussian.
!  The term with the m dof is not taken into account in this case
!  It is not needed Eq 2.58 Thomson and Chave

   !  Compute cross and eigenspectra

   do i = 1,nf
      do j = 1,kspec

         cross(i,j) = (wt_i(i,j)*wt_j(i,j)*yk_i(i,j)*conjg(yk_j(i,j)) )
	
         spec1(i,j) = wt_i(i,j) * abs(yk_i(i,j))**2
         spec2(i,j) = wt_j(i,j) * abs(yk_j(i,j))**2

      enddo
   enddo

   cross_all = sum(cross,dim=2)
   spec1_all = sum(spec1,dim=2)
   spec2_all = sum(spec2,dim=2)

   cohe_all  = abs(cross_all)**2/(spec1_all*spec2_all)
   phase_all = atan2(aimag(cross_all),real(cross_all))

   ! Compute the delete on estimates

   do i = 1,nf
      do j = 1,kspec

         cross(i,j) = cross_all(i) - cross(i,j)
         spec1(i,j) = spec1_all(i) - spec1(i,j)
         spec2(i,j) = spec2_all(i) - spec2(i,j)
 
         cohejk(i,j)  = abs(cross(i,j))**2/(spec1(i,j)*spec2(i,j))
         phasejk(i,j) = cross(i,j)/abs(cross(i,j))   ! Eq 2.61

         Q(i,j)       = atanh2(sqrt(cohejk(i,j))) 

      enddo
      
      qval(i)   = atanh2(sqrt(cohe(i))) 
      qmean(i)  = sum(Q(i,:))/real(kspec)
      phmean(i) = sum(phasejk(i,:))/real(kspec)       ! Eq 2.62

   enddo

   !  Statistics on the Q and phase variables

   do i = 1,nf
      do j = 1,kspec
      
         qvarjk(i,j)   = (Q(i,j) - qmean(i))**2

      enddo

      qvar(i)  = sum(qvarjk(i,:)) * real(kspec-1)/real(kspec)
      phvar(i) = 2.*(real(kspec-1)) * (1.-abs(phmean(i)))   ! Eq 2.63

   enddo

   do i = 1,nf
   
      qtop(i) = (qval(i) + 1.96*sqrt(qvar(i)))
      qbot(i) = (qval(i) - 1.96*sqrt(qvar(i)))

      if (qbot(i)<0.) then
         qbot(i)=0.
      endif
   
      cohe_ci(i,1) = (tanh(qbot(i)))**2
      cohe_ci(i,2) = (tanh(qtop(i)))**2

      phase_ci(i,1) = phase(i) - 1.96*sqrt(phvar(i))
      phase_ci(i,2) = phase(i) + 1.96*sqrt(phvar(i))
 
   enddo

   phase = phase * (180.0/3.1415926535)
   phase_ci = phase_ci * (180.0/3.1415926535)

end subroutine mtcohe

