subroutine qi(npts,dt,x,tbp,kspec,nf,nbl,freq,spec,sbar,B,bl,gl)

!
!  Compute the stationari QI basis functions and eigenvalues. 
!  Output returns the bl eigencoefficients which can be used 
!  to obtain the frequency derivative of the spectrum.
!  We follow
!  D. J. Thomson Quadratic-inverse spectrum estimates,
!  applications to paleoclimatology. Phil. Trans. Royal Soc.
!  Lond. A. no 332 pp 539-597, 1990
!
!  Last Modified 
!	German A. Prieto
!	20 December 2005
!
!  INPUT
!
!     npts		number of points in time series
!     dt		sampling rate
!     x(npts)		the time series (real)
!     tbp		the time-bandwidth product
!     kspec		number of tapers to use
!     nf		number of frequency points (npts/2+1)
!     nbl		number of QI coefficients to get
!
!  OUTPUT
!  
!     freq(nf)		frequency vector
!     spec(nf)		the spectrum estimate
!     sbar(nf)		the mean spectrum
!     B(513,nbl)	the QI eigenfunctions
!     bl(nf,nbl)	the QI eigencoefficients
!     gl(nbl)		the QI eigenvalues of B
!
!  calls mtspec, dpss, rsm_eig. pk_kernel, get_quad
!

!********************************************************************

   use spectra

   implicit none

!  Input

   integer, intent(in)	:: npts, kspec, nf, nbl

   real(4), intent(in)  :: dt, tbp

   real(4), dimension(npts), intent(in) :: x

!  Output

   real(4), dimension(nf), intent(out)       :: freq, spec, sbar
   real(4), dimension(513,nbl), intent(out)  :: B
   real(4), dimension(nf,nbl), intent(out)   :: bl
   real(4), dimension(nbl), intent(out)      :: gl

!  Other

   integer :: i, j, n

   integer, parameter :: lomx=256, nmx=2*lomx+1
   real(8), parameter :: pi=3.141592653589793d0, tpi = 2.d0*pi
   
   real(8) :: w, scl

!  Time series and spectrum

   complex(4), dimension(npts,kspec) ::  yk 

   real(4), dimension(nf,kspec) :: wt

!  DPSS sequences

!  Dpss 

   real(8), dimension(kspec)      :: lambda, theta
   real(8), dimension(npts,kspec) :: vn

!  Gaussian Quadrature and Abscissas

   real(8), dimension(nmx) :: wj, xloc

!  xint looping

   complex(8), dimension(nmx,nmx)  :: Pk
   real(8), dimension(nmx,nmx)     :: Kpk

   complex(8), dimension(nmx,kspec) :: dpsw

!  Bl matrix

   integer :: l, k
   complex(4), dimension(nbl,kspec,kspec) :: Gam_ljk
   real(4), dimension(nbl)   :: trace

!  Bl eigenfunctions

   real(8), dimension(nmx,nbl) :: B8
   real(8), dimension(nbl) :: gl8

!  bl eigencoefficients

   complex(4), dimension(nf,kspec) :: xk

   complex(4), dimension(nf,nbl) :: bl_cmp

!********************************************************************

!  Get the spectrum estimate

   call mtspec(npts,dt,x,tbp,kspec,nf,freq,spec,   &
                yk=yk, wt=wt, sbar=sbar )

!
!  Get the dpss
!

   call dpss(npts,dble(tbp),kspec,vn,lambda,theta)

   write(6,'(3x,a/(4f18.14))')'Prolate spheroidal eigenvalues:',  &
     			       (lambda(j),j=1,kspec)

   w  = dble(tbp)/dble(npts)

   call get_quad (npts, tbp, wj, xloc )

   call pk_kernel (npts, tbp, kspec, vn, lambda, xloc, wj, Pk, Kpk, dpsw)

   call rsm_eig( nmx, Kpk, nbl, gl8, B8)

   write(6,'(3x,a/(4f18.6))')' Projection Operator Pk eigenvalues:',  &
        			       (gl8(j),j=1,nbl)

!  Normalize so that int(-W,W) B**2 = 2W

   do l = 1,nbl
      scl     = sum(B8(:,l) * B8(:,l)) * (2.d0*w)
      B8(:,l) = B8(:,l) * sqrt((2.d0*w)/scl)
   enddo 
   do l = 1,nbl
      B8(:,l) = B8(:,l)/sqrt(wj) 
   enddo

   B = real(B8)

!  Put positive standard
  
   do i = 1,nbl
      if (B((nmx+1)/2 + 1,i) < 0. ) then
         B(:,i) = -B(:,i)
      endif
   enddo

!  Create the [Gamma_l]jk matrices B^l_jk from Thomson 1990
!  Note that this matrix is hermitian.
!  Test trace orthogonality

   do l = 1,nbl
      do j = 1,kspec
         do k = 1,kspec
	    Gam_ljk(l,j,k) = 0.0
	    do n = 1,nmx
               Gam_ljk(l,j,k) = Gam_ljk(l,j,k) 			   & 
	       		     +	(  real(w)			   &
	                     *  (  real(wj(n)) * (cmplx(dpsw(n,j)  &
	                     * conjg(dpsw(n,k)) ) * B(n,l)) ) )
            enddo
         enddo
      enddo
   enddo

! Trace of the matrices

   trace = 0.0
   do l = 1,nbl
      do j = 1,kspec
         do k = 1,kspec
            trace(l) = trace(l) + Gam_ljk(l,j,k)*Gam_ljk(l,k,j)
         enddo
      enddo
   enddo

   write(6,'(3x,a/(4f18.6))')' Gamma matrix eigenvalues:',  &
        			       (trace(j),j=1,nbl)

   gl = trace

!
!  Create the QI coefficients b_l
!

   ! Create the weigthed xk = dk*yk

   do k = 1,kspec
      xk(:,k) = wt(:,k)*yk(1:nf,k)
   enddo

   do l = 1,nbl
      do k = 1,nf

         bl_cmp(k,l) = sum ( matmul( conjg(xk(k,:)),Gam_ljk(l,:,:) ) &
	                 * xk(k,:) ) * 1./gl(l)

      enddo
   enddo

   bl = real(bl_cmp)



end subroutine qi

!--------------------------------------------------------------------
! Quadrature weights and abscissas
!--------------------------------------------------------------------

subroutine get_quad (npts, tbp, wj, xloc )

!
!  Gets the weights and abscissas of the Chebychev Gaussian
!  Quadrature, based on Gleen Ierley's codes. 
!  The weights and abscissas are then double sided so that 
!  it includes the whole range of integration of interest. 
!  The abscissas are then multiplied by the limits of 
!  integration (here assuming [-W, W], given by the bandwidth
!  product.
!
!  Input:
!	npts		number of data points of dpss
!	tbp		time bandwidth product
!
!  Output
!	wj(2*lomx+1)	weights of the Quadrature
!	xloc(2*lomx+1)	abscissas of the quadrature, in freq. units
!
!  calls set_xint
!

!********************************************************************

   implicit none

   integer, parameter :: nomx=8, lomx = 256

!  Input Output

   integer, intent(in) :: npts
   real(4), intent(in) :: tbp
   real(8), dimension(2*lomx+1), intent(out) :: wj, xloc

   real(8) :: w
   
   real(8), dimension(nomx,lomx+1) :: wi
   real(8), dimension(lomx+1)   :: xi

   integer :: i

!********************************************************************

!
! Get the weights and abscissas
!

   call set_xint(1,wi,xi)

!  Bandwidth W

   w = dble(tbp)/dble(npts)

!
!  Create the full weight and abcissa vectors
!

   do i = 1,2*lomx+1
      if (i>lomx+1) then
         xloc(i) = -w + (2.d0*w)*xi(2*lomx+1-(i-1))
         wj(i)   = wi(8,2*lomx+1-(i-1))
      elseif (i<lomx+1) then
         xloc(i) = w - (2.d0*w)*xi(i)
         wj(i)   = wi(8,i)
      elseif (i==lomx+1) then
         xloc(i) = w - (2.d0*w)*xi(i)
         wj(i)   = wi(8,i)*2.d0
      endif
   enddo

end subroutine get_quad

!--------------------------------------------------------------------

subroutine pk_kernel (npts, tbp, kspec, vn, lambda, xloc, wj, pk, K, dpsw)

!
!  This subroutine is in charge of computing the Pk frequency
!  projection kernel, as shown in Thomson (1990), at the frequencies
!  given by xloc.
!  xloc is generally the abscissas of a Quadrature rule, and the wj's
!  are the correspondent weights. 
!  This subroutine also computes the symmetric Kernel K, which is 
!  used to compute eigenvalues and eigenfunctions. This kernel is 
!  computed following the basic idea of Gaussian Quadrature to solve
!  an eigenfunction integral equation numerically, where in this 
!  case the Pk matrix is the convolution operator. 
!  
!  calls sft
!

!********************************************************************

   implicit none

   integer, parameter :: lomx=256
   real(8), parameter :: pi=3.141592653589793d0, tpi = 2.d0*pi

!  Input Output

   integer, intent(in) :: npts, kspec
   
   real(4), intent(in) :: tbp
   real(8), dimension(kspec), intent(in)      :: lambda
   real(8), dimension(npts,kspec), intent(in) :: vn

   real(8), dimension(2*lomx+1), intent(in) :: wj, xloc

   complex(8), dimension(2*lomx+1,2*lomx+1), intent(out)  :: Pk
   real(8), dimension(2*lomx+1,2*lomx+1), intent(out)     :: K

   complex(8), dimension(2*lomx+1,kspec), intent(out) :: dpsw

   real(8) :: w

!  SFT

   real(8) :: ct, st, om
   real(8), dimension(npts) :: dpss

!  Kernel looping

   integer :: i,j

   complex(8), dimension(2*lomx+1,kspec) :: fv

   real(8) :: scl

!********************************************************************   

!  The bandwidth

   w = dble(tbp)/dble(npts)

!
!  Get the DPSF
!

   do j = 1,kspec
      do i = 1,2*lomx+1

         om = tpi* xloc(i)
       
         dpss = vn(:,j)
         call sft(dpss,npts,om,ct,st)

         ! Normalize the sft
         ct = ct/sqrt(dble(npts))
         st = st/sqrt(dble(npts))

         fv(i,j)  = 1.d0/sqrt(lambda(j)) * cmplx(ct,st,kind=8)

     enddo
   enddo

!  Normalize int (-W,W) |1/sqrt(lambda_k) Vk|^2 df = 1

   do j = 1,kspec
      scl = w * real(sum(fv(:,j) * conjg(fv(:,j)) * wj(:)))
      fv(:,j) = fv(:,j)/sqrt(scl)
   enddo

!  Do the Pk(f,g) matrix
!  Do the Symmetric kernel

   do i = 1, 2*lomx+1

      do j = 1, 2*lomx+1

         Pk(i,j) = sum( fv(i,:) * conjg(fv(j,:)) )

         K(i,j)  = sqrt(wj(i)*wj(j)) * (abs(Pk(i,j)))**2

      enddo

   enddo

   dpsw = fv

end subroutine pk_kernel

!--------------------------------------------------------------------











